program main
    use graphics
    use util
    implicit none
    integer, parameter :: n = 5

    real :: x(n),y(n),vx(n),vy(n),m(n),r(n) ! posición, velocidad, masa y radio
    integer :: cr(n),cg(n),cb(n)            ! color
    real :: dt = 1

    real :: ux(n),uy(n)
    real :: aux
    integer :: k

    real :: maxx = 800, maxy = 600
    call mainprogram

contains

subroutine mainprogram

    call pizarra(nint(maxx),nint(maxy),"choques elásticos"//char(0))
    call escala(0.,maxx,0.,maxy)
    call fondo(0,32,0)
    call init_random_seed
    
    x  = [(50+100*k,k=1,n)]
    y  = randomsr(n,100.,200.)
    vx = randomsr(n,0.,2.)
    vy = randomsr(n,0.,2.)
    r  = [(15+k*2, k=1,n)]
    r(n) = 40
    m = r**3
    cr = 255
    cg = 255-[(k*30,k=1,n)]
    cb = 0
    cg(n) = 0

    call animate(run,5)

end

    subroutine run(tecla,time)
        integer :: tecla, time, k
        if(tecla/=0 .or. time > 3000) stop
        do k=1,n
            call choquepared(k)
            call choquebola(k)
            call color(0,32,0)
            call circulo(x(k),y(k),r(k))
            call mueve(k)
            call color(cr(k),cg(k),cb(k))
            call circulo(x(k),y(k),r(k))
        end do    
    end subroutine


    subroutine choquepared(k)
        integer :: k
        aux = x(k) + vx(k)*dt         
        if(aux<r(k) .or. aux>maxx-r(k)) vx(k) = -vx(k)
        aux = y(k) + vy(k)*dt
        if(aux<r(k) .or. aux>maxy-r(k)) vy(k) = -vy(k)
     end subroutine

    function toca(k,l)
        integer :: k,l
        logical :: toca
        toca = norm(sig(k)-sig(l)) < r(k)+r(l)
    end function

    function sig(k)
        integer :: k
        real :: sig(2)
        sig = [x(k)+vx(k)*dt, y(k)+vy(k)*dt]
    end function

    subroutine choquebola(k)
        integer :: k, l
        do l = 1,n
            if(k/=l .and. toca(k,l)) then
               call choque(k,l,1.0)
               call choque(l,k,1.0)
               vx(k) = ux(k); vy(k) = uy(k)
               vx(l) = ux(l); vy(l) = uy(l)
            end if
        end do   
    end subroutine

    subroutine choque(j,k,f)
        integer :: j, k
        real   :: gx, gy, n, ty, wx, wy, f
        gx = x(j)-x(k)
        gy = y(j)-y(k)
        n = sqrt(gx**2+gy**2)
        gx = gx/n
        gy = gy/n
        
        wx = -gy*vx(j) + gx*vy(j)
        wy =  gx*vx(j) + gy*vy(j)
        ty =  gx*vx(k) + gy*vy(k)
        
        wy = f*(m(j)*wy+m(k)*ty + m(k)*(ty-wy))/(m(j)+m(k))
        
        ux(j) = -gy*wx + gx*wy
        uy(j) =  gx*wx + gy*wy
    end subroutine

    subroutine mueve(k)
        integer :: k
        x(k) = x(k) + vx(k)*dt
        y(k) = y(k) + vy(k)*dt
    end subroutine
END

