module graphics
    use iso_c_binding, only: c_int, c_double
    implicit none

    real(kind=c_double), bind(C, name="mouse_x")   :: mouse_x
    real(kind=c_double), bind(C, name="mouse_y")   :: mouse_y
    integer(kind=c_int), bind(C, name="mouse_col") :: mouse_col
    integer(kind=c_int), bind(C, name="mouse_row") :: mouse_row

    integer(kind=c_int), bind(C, name="lastkey")   :: lastkey

    real, parameter :: pi =4.*atan(1.0)

    integer, parameter             &
       :: arrow_up    = 1073741906 &
        , arrow_down  = 1073741905 &
        , arrow_left  = 1073741904 &
        , arrow_right = 1073741903 &
        , escape_key  = 27         &
        , plus_key    = 1073741911 &
        , minus_key   = 1073741910 &
        , mouse       = -1         &
        , wheel_up    = -2         &
        , wheel_down  = -3
interface

    SUBROUTINE raw_pizarra(w,h,title) bind(C, name="pizarra")
        use iso_c_binding, only: c_int, c_char
        integer(kind=c_int), value :: w,h
        character(kind=c_char) :: title(*)
    END SUBROUTINE

    SUBROUTINE escala(x1,x2,y1,y2) bind(C, name="escala")
        use iso_c_binding, only: c_double
        real(kind=c_double), value :: x1, y1, x2, y2
    END SUBROUTINE

    SUBROUTINE autoescala(x1,x2,yc) bind(C, name="autoescala")
        use iso_c_binding, only: c_double
        real(kind=c_double), value :: x1, x2, yc
    END SUBROUTINE

    SUBROUTINE ejes(b,s) bind(C, name="ejes")
        use iso_c_binding, only: c_double
        real(kind=c_double), value :: b,s
    END SUBROUTINE

    SUBROUTINE color(r,g,b) bind(C, name="color")
        use iso_c_binding, only: c_int
        integer(kind=c_int), value :: r,g,b
    END SUBROUTINE

    SUBROUTINE fondo(r,g,b) bind(C, name="fondo")
        use iso_c_binding, only: c_int
        integer(kind=c_int), value :: r,g,b
    END SUBROUTINE

    SUBROUTINE recta(x1,y1,x2,y2) bind(C, name="recta")
        use iso_c_binding, only: c_double
        real(kind=c_double), value :: x1, y1, x2, y2
    END SUBROUTINE

    SUBROUTINE rectangulo(x1,y1,x2,y2) bind(C, name="rectangulo")
        use iso_c_binding, only: c_double
        real(kind=c_double), value :: x1, y1, x2, y2
    END SUBROUTINE

    SUBROUTINE trigono(x1,y1,x2,y2,x3,y3) bind(C, name="trigon")
        use iso_c_binding, only: c_double
        real(kind=c_double), value :: x1, y1, x2, y2, x3, y3
    END SUBROUTINE

    SUBROUTINE punto(x,y) bind(C, name="punto")
        use iso_c_binding, only: c_double
        real(kind=c_double), value :: x,y
    END SUBROUTINE

    SUBROUTINE pixel(x,y,r,g,b) bind(C, name="rpixel")
        use iso_c_binding, only: c_int
        integer(kind=c_int), value :: x,y,r,g,b
    END SUBROUTINE

    SUBROUTINE rectaA(x,y) bind(C, name="rectaa")
        use iso_c_binding, only: c_double
        real(kind=c_double), value :: x,y
    END SUBROUTINE

    SUBROUTINE avanza(d) bind(C, name="avanza")
        use iso_c_binding, only: c_double
        real(kind=c_double), value :: d
    END SUBROUTINE

    SUBROUTINE gira(a) bind(C, name="gira")
        use iso_c_binding, only: c_double
        real(kind=c_double), value :: a
    END SUBROUTINE


    SUBROUTINE circulo(x,y,r) bind(C, name="circulo")
        use iso_c_binding, only: c_double
        real(kind=c_double), value :: x,y,r
    END SUBROUTINE

    SUBROUTINE circunferencia(x,y,r) bind(C, name="circunferencia")
        use iso_c_binding, only: c_double
        real(kind=c_double), value :: x,y,r
    END SUBROUTINE

    SUBROUTINE raw_texto(x,y,s) bind(C, name="texto")
        use iso_c_binding, only: c_double, c_char
        real(kind=c_double), value :: x,y
        character(kind=c_char) :: s(*)
    END SUBROUTINE

    SUBROUTINE raw_textow(x,y,s) bind(C, name="textow")
        use iso_c_binding, only: c_int, c_char
        integer(kind=c_int), value :: x,y
        character(kind=c_char) :: s(*)
    END SUBROUTINE

    function xcol(c) bind(C, name="xcol")
        use iso_c_binding, only: c_double, c_int
        real(c_double) :: xcol
        integer(c_int), value :: c
    end function

    function yrow(r) bind(C, name="yrow")
        use iso_c_binding, only: c_double, c_int
        real(c_double) :: yrow
        integer(c_int), value :: r
    end function

    subroutine frame_skip(s) bind(C, name="set_frame_skip")
        use iso_c_binding, only: c_int
        integer(kind=c_int), value :: s
    end subroutine

    subroutine start_paused() bind(C, name="set_step")
    end subroutine

    subroutine saveimage(n) bind(C, name="save_image")
        use iso_c_binding, only: c_int
        integer(kind=c_int), value :: n
    end subroutine

    SUBROUTINE raw_load_image(filename) bind(C, name="load_image")
        use iso_c_binding, only: c_char
        character(kind=c_char) :: filename(*)
    END SUBROUTINE

    SUBROUTINE heatcolor(x,xmin,xzero,xmax) bind(C, name="heatcolor")
        use iso_c_binding, only: c_double
        real(kind=c_double), value :: x,xmin,xzero,xmax
    END SUBROUTINE

    SUBROUTINE graycolor(x,xmin,xmax) bind(C, name="graycolor")
        use iso_c_binding, only: c_double
        real(kind=c_double), value :: x,xmin,xmax
    END SUBROUTINE

    function modif_state(k) bind(C, name="modif_state")
        use iso_c_binding, only: c_int
        integer(kind=c_int), value :: k
        integer(kind=c_int) :: modif_state
    end function

    subroutine dibuja(f) bind(C, name="dibuja")
        interface
            subroutine f(tecla) bind(C)
                use iso_c_binding, only: c_int
                integer(kind=c_int) :: tecla
            end subroutine
        end interface
    end subroutine

    subroutine animate(f,t) bind(C, name="animate")
        use iso_c_binding, only: c_int
        integer(kind=c_int), value :: t
        interface
            subroutine f(tecla,time) bind(C)
                use iso_c_binding, only: c_int
                integer(kind=c_int) :: tecla, time
            end subroutine
        end interface
    end subroutine

    subroutine onzoom(f) bind(C, name="onzoom")
        interface
            subroutine f() bind(C)
            end subroutine
        end interface
    end subroutine

end interface

contains

    subroutine texto(x,y,s)
        real :: x,y
        character(*) :: s
        call raw_texto(x,y,s//char(0))
    end

    subroutine textow(x,y,s)
        integer :: x,y
        character(*) :: s
        call raw_textow(x,y,s//char(0))
    end

    subroutine pizarra(w,h,s)
        integer :: w,h
        character(*) :: s
        call raw_pizarra(w,h,s//char(0))
    end

    subroutine loadimage(s)
        character(*) :: s
        call raw_load_image(s//char(0))
    end

    subroutine plot(x,y)
        real :: x(:), y(:)
        integer :: k
        call punto(x(1),y(1))
        do k=2,size(x)
            call rectaa(x(k),y(k))
        end do
    end subroutine

    function control_key()
        logical :: control_key
        control_key = modif_state(1) == 1
    end function

    function shift_key()
        logical :: shift_key
        shift_key = modif_state(2) == 1
    end function

    function alt_key()
        logical :: alt_key
        alt_key = modif_state(3) == 1
    end function

    subroutine zoomcontrol(tecla)
        integer :: tecla
        select case(tecla)
            case (wheel_up)
                call zoom(1.2)
            case (wheel_down)
                call zoom(1/1.2)
            case (mouse)
                call zoomcenter(mouse_x,mouse_y)
        end select        
    end subroutine

end module

